﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Reflection
Imports System.Text
Imports Microsoft.Office.Interop.Word
Imports Microsoft.ReportingServices.Interfaces
Imports Microsoft.ReportingServices.OnDemandReportRendering
Imports System.Configuration
Imports System.Diagnostics


Public Class CustomWordRenderer
    Implements IExtension, IRenderingExtension

#Region "IExtension Members"
    Public ReadOnly Property LocalizedName() As String Implements IExtension.LocalizedName
        Get
            Return "CUSTOM WORD"
        End Get
    End Property

    Public Sub SetConfiguration(ByVal configuration As String) Implements IExtension.SetConfiguration

    End Sub

#End Region

#Region "IRenderingExtension Members"

    Public Sub GetRenderingResource(ByVal createAndRegisterStreamCallback As CreateAndRegisterStream, ByVal deviceInfo As System.Collections.Specialized.NameValueCollection) Implements IRenderingExtension.GetRenderingResource

    End Sub

    Public Function IsProcessOpen(ByVal name As String) As Boolean
        For Each clsProcess As Process In Process.GetProcesses()

            If clsProcess.ProcessName.ToLower().Contains(name) Then
                Return True
            End If
        Next clsProcess
        Return False
    End Function

    Public Function Render(ByVal report As Microsoft.ReportingServices.OnDemandReportRendering.Report, ByVal reportServerParameters As System.Collections.Specialized.NameValueCollection, ByVal deviceInfo As System.Collections.Specialized.NameValueCollection, ByVal clientCapabilities As System.Collections.Specialized.NameValueCollection, ByRef renderProperties As System.Collections.Hashtable, ByVal createAndRegisterStream As CreateAndRegisterStream) As Boolean Implements IRenderingExtension.Render
        Dim source As String = "Custom Rendering Extension"
        Dim message As String = String.Empty
        Dim userName As String = String.Empty

        Try
            ' Get the rerport URL
            Dim downloadUrl As String = String.Concat(report.GetReportUrl(False), "&rs:Format=WORD&rs:Command=Render&rc:FileExtension=doc")

            ' Concatenate parameters to report URL
            For Each parameter As ReportParameter In report.Parameters
                If parameter.MultiValue Then
                    For i As Integer = 0 To parameter.Instance.Values.Count - 1
                        downloadUrl = String.Concat(downloadUrl, "&", parameter.Instance.Prompt, "=", parameter.Instance.Values(i).ToString())
                    Next i
                Else
                    If Not String.IsNullOrEmpty(parameter.Instance.Prompt) Then
                        downloadUrl = String.Concat(downloadUrl, "&", parameter.Instance.Prompt, "=", parameter.Instance.Value)
                    End If
                End If
            Next parameter

            ' Get File Paths from device info configuration
            Dim sourceHeaderfileName As Object = deviceInfo("SourceHeaderFileName").ToString()
            Dim bodyFileName As String = Path.GetFileNameWithoutExtension(deviceInfo("SourceBodyFileName").ToString())
            Dim timeStamp As String = Date.Now.ToFileTime().ToString()
            bodyFileName &= timeStamp
            Dim bodyFileExtension As String = Path.GetExtension(deviceInfo("SourceBodyFileName").ToString())
            Dim bodyFilePath As String = Path.GetDirectoryName(deviceInfo("SourceBodyFileName").ToString())
            Dim sourceBodyfileName As Object = String.Format("{0}\{1}{2}", bodyFilePath, bodyFileName, bodyFileExtension)

            Dim mergedFileName As String = Path.GetFileNameWithoutExtension(deviceInfo("MergedFileName").ToString())
            mergedFileName &= timeStamp
            Dim mergedFilePath As String = Path.GetDirectoryName(deviceInfo("MergedFileName").ToString())
            Dim mergedFile As Object = String.Format("{0}\{1}{2}", mergedFilePath, mergedFileName, bodyFileExtension)
            EventLog.WriteEntry(source, mergedFile.ToString(), EventLogEntryType.Information)

            Dim doc() As Byte

            ' Download the report rendered in word format
            Dim webClient As New WebClient()
            webClient.UseDefaultCredentials = True
            doc = webClient.DownloadData(downloadUrl)

            ' Write the downloaded document to disc
            Dim fs As New FileStream(sourceBodyfileName.ToString(), FileMode.Create)
            Dim bw1 As New BinaryWriter(fs)
            bw1.Write(doc)
            bw1.Flush()
            bw1.Close()
            fs.Close()

            ' Initialize objects for word automation
            Dim missing As New Object
            missing = missing.Value
            Dim readOnlyFalse As Object = False
            Dim readOnlyTrue As Object = True
            Dim isVisible As Object = False
            Dim saveChanges As Object = True
            Dim oTrue As Object = True
            Dim styleHeading1 As Object = Microsoft.Office.Interop.Word.WdBuiltinStyle.wdStyleHeading1
            Dim styleHeading2 As Object = Microsoft.Office.Interop.Word.WdBuiltinStyle.wdStyleHeading2



            ' Application Object
            Dim wordApp As Microsoft.Office.Interop.Word.Application = Nothing

            ' Document Objects
            Dim sourceWordDocumentHeader As Microsoft.Office.Interop.Word.Document
            Dim sourceWordDocumentBody As Microsoft.Office.Interop.Word.Document
            Dim mergedWordDocument As Microsoft.Office.Interop.Word.Document

            ' Launch the word Application
            wordApp = New Microsoft.Office.Interop.Word.Application()

            ' Hide the User Interface
            wordApp.Visible = False

            ' Open the Main Body Document
            sourceWordDocumentBody = wordApp.Documents.Open(sourceBodyfileName, missing, readOnlyFalse, missing, missing, missing, missing, missing, missing, missing, missing, isVisible, missing, missing, missing, missing)

            ' Open the header document as read only
            sourceWordDocumentHeader = wordApp.Documents.Open(sourceHeaderfileName, missing, readOnlyTrue, missing, missing, missing, missing, missing, missing, missing, missing, isVisible, missing, missing, missing, missing)

            ' Create a new document to store the merged output
            mergedWordDocument = wordApp.Documents.Add(missing, missing, missing, missing)

            ' Merge the header and body contents and paste in the merged document
            Dim range As Object = mergedWordDocument.Range(missing, missing)
            Dim headerSection As Section = mergedWordDocument.Sections.First
            headerSection.PageSetup.Orientation = WdOrientation.wdOrientPortrait
            Dim sourceHeaderRange As Range = sourceWordDocumentHeader.Range(missing, missing)
            sourceHeaderRange.Copy()
            headerSection.Range.Paste()
            Dim bodySection As Section = mergedWordDocument.Sections.Add(missing, missing)
            bodySection.PageSetup.Orientation = WdOrientation.wdOrientLandscape
            Dim sourcebodyRange As Range = sourceWordDocumentBody.Range(missing, missing)
            sourcebodyRange.Copy()
            bodySection.Range.Paste()

            ' Save merged document
            mergedWordDocument.SaveAs(mergedFile, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing)


            ' Convert Table Entries to Proper Heading for TOC
            For Each myField As Microsoft.Office.Interop.Word.Field In mergedWordDocument.Fields
                Dim rngFieldCode As Microsoft.Office.Interop.Word.Range = myField.Code
                Dim fieldText As String = rngFieldCode.Text

                If fieldText.StartsWith(" TC") Then
                    Dim [end] As Int32 = fieldText.IndexOf("""", 5)
                    Dim fieldNameLength As Int32 = [end] - 5
                    Dim fieldName As String = fieldText.Substring(5, fieldNameLength)
                    rngFieldCode.Style = styleHeading1

                    ' Clear the TC tag
                    myField.Copy()
                    myField.Delete()
                End If
            Next myField

            ' Update TOC
            mergedWordDocument.TablesOfContents(1).Update()

            ' Save Merged Document
            mergedWordDocument.SaveAs(mergedFile, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing)

            ' Close document and application
            mergedWordDocument.Close(missing, missing, missing)
            sourceWordDocumentHeader.Close(missing, missing, missing)
            sourceWordDocumentBody.Close(missing, missing, missing)
            wordApp.Quit(missing, missing, missing)

            ' Stream the document
            Dim s As Stream = createAndRegisterStream("Monthly Website Usage Report", "doc", Encoding.Default, "application/msword", True, StreamOper.CreateAndRegister)

            Dim bw As New BinaryWriter(s)
            Dim byteArray() As Byte = File.ReadAllBytes(mergedFile.ToString())
            bw.Write(byteArray)
            bw.Flush()

            Return True
        Catch ex As Exception
            message = ex.ToString()

            ' Write exception to event log
            EventLog.WriteEntry(source, message, EventLogEntryType.Error)

            ' Report rendering failed
            Return False
        End Try
    End Function

    Public Function RenderStream(ByVal streamName As String, ByVal report As Microsoft.ReportingServices.OnDemandReportRendering.Report, ByVal reportServerParameters As System.Collections.Specialized.NameValueCollection, ByVal deviceInfo As System.Collections.Specialized.NameValueCollection, ByVal clientCapabilities As System.Collections.Specialized.NameValueCollection, ByRef renderProperties As System.Collections.Hashtable, ByVal createAndRegisterStream As CreateAndRegisterStream) As Boolean Implements IRenderingExtension.RenderStream
        Return True
    End Function

#End Region
End Class