﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.IO
Imports System.Data.SqlClient
Imports System.Data
Imports Microsoft.SqlServer.Server
Imports System.Reflection


Public Class WebSiteUsersChart
    Private bitmap As Bitmap
    Private graphics As Graphics
    Private chartWidth As Integer
    Private chartHeight As Integer
    Private borderPadding As Integer
    Private axisBottomPadding As Integer
    Private axisHorizontalPadding As Integer
    Private penWidth As Integer
    Private barPadding As Integer
    Private barBottomPadding As Integer
    Private barDataFontSize As Integer
    Private xAxisLabels As List(Of String)
    Private dataList As List(Of Integer)
    Private chartTitle As String
    Private continent As String
    Private month As Integer
    Private year As Integer

    Public Sub New(ByVal chartWidth As Integer, ByVal chartHeight As Integer, ByVal chartTitle As String, ByVal continent As String, ByVal month As Integer, ByVal year As Integer)
        Me.chartHeight = chartHeight
        Me.chartWidth = chartWidth
        Me.chartTitle = chartTitle
        Me.continent = continent
        Me.month = month
        Me.year = year


        Me.bitmap = New Bitmap(Me.chartWidth, Me.chartHeight)
        Me.graphics = graphics.FromImage(Me.bitmap)
        Me.graphics.SmoothingMode = SmoothingMode.AntiAlias
        Me.graphics.Clear(Color.White)
        Me.borderPadding = 55
        Me.axisBottomPadding = 30
        Me.axisHorizontalPadding = 10
        Me.penWidth = 2
        Me.barPadding = 20
        Me.barBottomPadding = 5
        Me.barDataFontSize = 10
    End Sub

    Public Function PlotChart() As Bitmap
        Me.GetChartData()

        ' If there is no data, return
        If Me.dataList.Count = 0 Then
            Return Nothing
        End If

        ' Load Face image
        Dim faceImage As Image = Me.GetEmbeddedImage("WebSiteUsersChartCore.Images.Face.png")

        ' Draw area border
        Dim border As New Rectangle(Me.penWidth, Me.penWidth, Me.chartWidth - Me.penWidth * 2, Me.chartHeight - Me.penWidth * 2)
        Me.graphics.DrawRectangle(New Pen(Color.Black, Me.penWidth), border)

        ' Fill background
        Me.graphics.FillRectangle(New LinearGradientBrush(border, Color.FromArgb(255, 255, 255), Color.FromArgb(219, 231, 245), LinearGradientMode.Vertical), border)

        ' Calculate chart area
        Dim chartArea As New Rectangle(Me.penWidth, Me.borderPadding + Me.penWidth, Me.chartWidth - Me.penWidth * 2, Me.chartHeight - Me.borderPadding - Me.penWidth * 2)

        ' Draw Chart Titile
        Dim dateTime As New Date(year, month, 15)
        Dim consolidatedChartTitle As String = String.Format("{0} ({1}) {2} {3}", chartTitle, continent, dateTime.ToString("MMMM"), year)
        Me.graphics.DrawString(consolidatedChartTitle, New Font("Calibri", 12, FontStyle.Bold), Brushes.Black, 10, 10)

        ' Draw x-axis
        Me.graphics.DrawLine(New Pen(Color.Black, Me.penWidth), chartArea.Left + Me.axisHorizontalPadding, chartArea.Bottom - Me.axisBottomPadding, chartArea.Right - Me.axisHorizontalPadding, chartArea.Bottom - Me.axisBottomPadding)
        Dim xAxisLength As Integer = (chartArea.Right - Me.axisHorizontalPadding) - (chartArea.Left + Me.axisHorizontalPadding)

        ' Draw x-axis Labels
        Dim barCount As Integer = xAxisLabels.Count
        Dim barWidth As Integer = xAxisLength \ barCount
        Dim nextLabelStartPoint As Integer = chartArea.Left + Me.axisHorizontalPadding
        For Each xAxisLabel As String In xAxisLabels
            Dim centerPoint As Integer = nextLabelStartPoint + barWidth \ 2
            Dim stringWidth As Single = Me.graphics.MeasureString(xAxisLabel, New Font("Calibri", 10, FontStyle.Bold)).Width
            Me.graphics.DrawString(xAxisLabel, New Font("Calibri", 10, FontStyle.Bold), Brushes.Black, centerPoint - stringWidth / 2, chartArea.Bottom - Me.axisBottomPadding)

            nextLabelStartPoint += barWidth
        Next xAxisLabel

        ' Draw Bars
        Dim maxValue As Integer = dataList.Max()
        Dim minValue As Integer = dataList.Min()
        Dim nextBarStartPoint As Integer = chartArea.Left + Me.axisHorizontalPadding
        Dim maxBarHeight As Integer = chartArea.Bottom - Me.axisBottomPadding - chartArea.Top
        For Each data As Integer In dataList
            Dim barHeight As Integer = (data * maxBarHeight \ maxValue) - Me.barDataFontSize
            Dim dataLabelPoint As Integer = nextBarStartPoint + barWidth \ 2 - 10

            ' Fill bar
            Dim barFillArea As New Rectangle(nextBarStartPoint + barPadding, chartArea.Top + (maxBarHeight - barHeight), barWidth - 2 * barPadding, barHeight - Me.barBottomPadding)
            Me.graphics.FillRectangle(New LinearGradientBrush(barFillArea, Color.FromArgb(202, 234, 247), Color.FromArgb(0, 175, 220), LinearGradientMode.Vertical), barFillArea)

            ' Draw data labels
            Me.graphics.DrawString(data.ToString(), New Font("Calibri", Me.barDataFontSize, FontStyle.Bold Or FontStyle.Italic), Brushes.Black, dataLabelPoint, chartArea.Top + (maxBarHeight - barHeight) - Me.barDataFontSize - 5)
            nextBarStartPoint += barWidth

            ' Draw Face image in bar
            Me.graphics.DrawImage(faceImage, New Rectangle(barFillArea.X, barFillArea.Y, barWidth - 2 * barPadding, (barWidth - 2 * barPadding) * 85 \ 73))
        Next data

        Return Me.bitmap
    End Function

    Private Sub GetChartData()
        Me.GetUserData()
    End Sub

    Private Sub GetUserData()
        dataList = New List(Of Integer)()
        xAxisLabels = New List(Of String)()

        ' Replace the connection string depending on wether testing using Test Harness or
        ' deploying the solution to SQL Server as a SQL CLR function
        'string connectionString = "context connection=true";
        Dim connectionString As String = "Data Source=.;Initial Catalog=WebsiteUsersReportDB; Integrated Security=sspi;"

        Using connection As New SqlConnection(connectionString)
            Using sqlCommand As New SqlCommand("dbo.usp_GetUsageData", connection)
                sqlCommand.CommandType = CommandType.StoredProcedure
                sqlCommand.Parameters.Add("@Month", SqlDbType.Int).Value = Me.month
                sqlCommand.Parameters.Add("@Year", SqlDbType.Int).Value = Me.year
                sqlCommand.Parameters.Add("@Region", SqlDbType.NVarChar).Value = continent
                connection.Open()
                Dim reader As SqlDataReader = sqlCommand.ExecuteReader()
                Do While reader.Read()
                    Me.xAxisLabels.Add(reader.GetString(reader.GetOrdinal("CountryName")))
                    Me.dataList.Add(reader.GetInt32(reader.GetOrdinal("UserCount")))

                Loop
            End Using
        End Using
    End Sub

    Private Function GetEmbeddedImage(ByVal name As String) As Image
        Dim [assembly] As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim imageStream As Stream = [assembly].GetManifestResourceStream(name)
        Return bitmap.FromStream(imageStream)
    End Function
End Class